/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.el;

import java.util.HashMap;
import java.util.Map;

import woolpack.TestBean;
import junit.framework.TestCase;

public class PropertyELTest extends TestCase {

	public void testConstructor(){
		try{
			new PropertyEL(null, ELConstants.PROPERTY_EL_FACTORY);
			fail();
		}catch(final NullPointerException expected){
		}
		try{
			new PropertyEL("", ELConstants.PROPERTY_EL_FACTORY);
			fail();
		}catch(final StringIndexOutOfBoundsException expected){
		}
		try{
			new PropertyEL("key0", null);
			fail();
		}catch(final NullPointerException expected){
		}
	}
	
	public void testNormalBean(){
		final TestBean bean = new TestBean();
		
		assertEquals(new Integer(0), new PropertyEL("myInt", ELConstants.PROPERTY_EL_FACTORY).getValue(bean));
		assertEquals("0", new PropertyEL("myInt", ELConstants.PROPERTY_EL_FACTORY).getValue(bean, String.class));
		new PropertyEL("myInt", ELConstants.PROPERTY_EL_FACTORY).setValue(bean, new Integer(2));
		new PropertyEL("myInt", ELConstants.PROPERTY_EL_FACTORY).setValue(bean, new Integer(1));
		assertEquals(new Integer(1), new PropertyEL("myInt", ELConstants.PROPERTY_EL_FACTORY).getValue(bean));

		try{
			new PropertyEL("yourInt", ELConstants.PROPERTY_EL_FACTORY).getValue(bean);
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
		try{
			new PropertyEL("yourInt", ELConstants.PROPERTY_EL_FACTORY).getValue(bean, String.class);
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
		try{
			new PropertyEL("yourInt", ELConstants.PROPERTY_EL_FACTORY).setValue(bean, new Integer(2));
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
	}
	
	public void testNormalMap(){
		final Map<String, Object> map = new HashMap<String,Object>();
		map.put("param0", new Integer(3));
		assertEquals(new Integer(3), new PropertyEL("param0", ELConstants.PROPERTY_EL_FACTORY).getValue(map));
		assertEquals("3", new PropertyEL("param0", ELConstants.PROPERTY_EL_FACTORY).getValue(map, String.class));
		
		new PropertyEL("param1", ELConstants.PROPERTY_EL_FACTORY).setValue(map, "value2");
		new PropertyEL("param1", ELConstants.PROPERTY_EL_FACTORY).setValue(map, "value1");
		assertEquals("value1", new PropertyEL("param1", ELConstants.PROPERTY_EL_FACTORY).getValue(map));

		assertNull(new PropertyEL("param2", ELConstants.PROPERTY_EL_FACTORY).getValue(map));

		new PropertyEL("param3", ELConstants.PROPERTY_EL_FACTORY).setValue(map, null);
		assertNull(new PropertyEL("param3", ELConstants.PROPERTY_EL_FACTORY).getValue(map));
	}
	
	public void testEquals(){
		{
			final PropertyEL obj = new PropertyEL("key0");
			assertTrue(obj.equals(obj));
		}
		assertEquals(new PropertyEL("key0"), new PropertyEL("key0"));
		assertFalse(new PropertyEL("key0").equals(new PropertyEL("key1")));
		assertFalse(new PropertyEL("key0").equals(new Object()));
		assertFalse(new PropertyEL("key0").equals(null));
	}
	
	public void testHashCode(){
		assertTrue(new PropertyEL("key0").hashCode() == new PropertyEL("key0").hashCode());
		assertTrue(new PropertyEL("key0").hashCode() != new PropertyEL("key1").hashCode());
	}
	
	public void testToString(){
		assertEquals(
				"woolpack.el.PropertyEL@@[propertyName=key0,factory=woolpack.el.MapPropertyELFactory@@[convertEL=woolpack.el.CollectionTypeConvertEL@@[convertEL=woolpack.el.SimpleTypeConvertEL@@],factory=woolpack.el.CachePropertyELFactory@@[factory=woolpack.el.ReflectionPropertyELFactory@@[convertEL=woolpack.el.CollectionTypeConvertEL@@[convertEL=woolpack.el.SimpleTypeConvertEL@@]]]]]", 
				new PropertyEL("key0").toString().replaceAll("@[a-z0-9]+", "@@"));
	}
}
