/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.validator;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Pattern;

import woolpack.TestUtils;
import woolpack.utils.MapBuilder;

import junit.framework.TestCase;

public class RegExpValidatorTest extends TestCase {

	public void testConstructor(){
		try{
			new RegExpValidator((Pattern)null);
			fail();
		}catch(final NullPointerException expected){
		}
		try{
			new RegExpValidator((String)null);
			fail();
		}catch(final NullPointerException expected){
		}
	}

	public void testInterpret(){
		final ValidatorExpression expression = new RegExpValidator("a[0-9]");
		final ValidatorContext context = new ValidatorContext();
		context.setInputMap(new HashMap<String,List<Object>>());
		context.setTmpKey("key0");
		
		context.setTmpValue("a4");
		assertTrue(expression.interpret(context));
		assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList("a4")).get(), context.getInputMap()));

		context.setTmpValue("aa");
		assertFalse(expression.interpret(context));
		assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList("aa")).get(), context.getInputMap()));
		
		try{
			context.setTmpValue(new Integer(1));
			expression.interpret(context);
			fail();
		}catch(final ClassCastException e){
			assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList(1)).get(), context.getInputMap()));
		}
		
		try{
			context.setTmpValue(null);
			expression.interpret(context);
			fail();
		}catch(final NullPointerException e){
			assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList((Object)null)).get(), context.getInputMap()));
		}
	}
	
	public void testAppendTo() throws IOException{
		final ValidatorExpression expression = new RegExpValidator("a[0-9]");
		final StringBuilder sb = new StringBuilder();
		expression.appendTo(sb);
		assertEquals("new RegExpValidator(\"a[0-9]\")", sb.toString());
	}
	
	public void testAddMessageTo(){
		final ValidatorExpression expression = new RegExpValidator("a[0-9]");
		final Collection<String> expectedCollection = new ArrayList<String>();
		expression.addMessageTo(expectedCollection);
		assertTrue(expectedCollection.isEmpty());
	}
}
